//  FindDialog.cpp version 1.5
//  yudit package - Unicode Editor for the X Window System (and Linux) 
//
//  Author: gsinai@iname.com (Gaspar Sinai)
//  GNU Copyright (C) 1997,1998,1999  Gaspar Sinai
// 
//  yudit version 1.5  Copyright(C) 30 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.4  Copyright(C) 25 November, 1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.3  Copyright(C)  5 April,    1999, Tokyo Japan  Gaspar Sinai
//  yudit version 1.2  Copyright(C) 10 December, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.1  Copyright(C) 23 August,   1998, Tokyo Japan  Gaspar Sinai
//  yudit version 1.0  Copyright(C) 17 May,      1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.99 Copyright(C)  4 April,    1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.97 Copyright(C)  4 February, 1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.95 Copyright(C) 10 January,  1998, Tokyo Japan  Gaspar Sinai
//  yudit version 0.94 Copyright(C) 17 December, 1997, Tokyo Japan  Gaspar Sinai
//  yudit version 0.9 Copyright (C)  8 December, 1997, Tokyo Japan  Gaspar Sinai
//  yutex version 0.8 Copyright (C)  5 November, 1997, Tokyo Japan  Gaspar Sinai
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//

#include <FindDialog.h>
#include <UFontFreeType.h>

#include <Translate.h>
#include <Xm/Xm.h>
#include <Xm/DialogS.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>
#include <Xm/RowColumn.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <X11/Intrinsic.h>
#include "MYudit.h"
#include <unistd.h>
#include <math.h>
#include <stdlib.h>
#include <strings.h>

FindDialog::FindDialog (Widget parentIn, MUWidget* uwidgetIn, const Config* configIn)
{
	const char*		string;
	const char*		okString;
	const char*		cancelString;
	const char*		helpString;


	topConfig = configIn;
	findResult = 2;
	findLabel = 0;
	replaceLabel = 0;
	findUWidget = 0;
	replaceUWidget = 0;
	findForm = 0;
	replaceForm = 0;
	rowColumn = 0;
	topForm = 0;
	rowColumn = 0;
	toplevel = 0;

	toplevel = XtVaCreatePopupShell ("FindShell",
		xmDialogShellWidgetClass, parentIn,
		XmNtitle, "Yudit: Find",
		0);

	okString = Translate ("Find");
	cancelString = Translate ("Find/Replace");
	helpString = Translate ("Dismiss");
	messageBox = XtVaCreateWidget ("FindDialog",
		xmMessageBoxWidgetClass, toplevel,
		XmNdialogType,	XmDIALOG_MESSAGE,
		XmNautoUnmanage, False,
		XmNdialogStyle, XmDIALOG_PRIMARY_APPLICATION_MODAL,
		XtVaTypedArg,	XmNokLabelString,		
		XmRString,	okString, strlen (okString) + 1,
		XtVaTypedArg,	XmNcancelLabelString,		
		XmRString,	cancelString, strlen (cancelString) + 1,
		XtVaTypedArg,	XmNhelpLabelString,		
		XmRString,	helpString, strlen (helpString) + 1,
		0);

	topForm = XtVaCreateManagedWidget ("FindTopForm",
		xmFormWidgetClass, messageBox, 0);

	rowColumn = XtVaCreateManagedWidget ("FindRowColumn",
		xmRowColumnWidgetClass, topForm,
		XmNpacking,     XmPACK_COLUMN,
		XmNnumColumns,	2,
		XmNorientation,	XmHORIZONTAL,
		XmNisAligned,	True,
		XmNentryAlignment,	XmALIGNMENT_END,

#if ((XmVERSION>=2) || (XmVERSION==1 && XmREVISION>=2))
		XmNentryVerticalAlignment,	XmALIGNMENT_CONTENTS_TOP,
#endif
		0);	
	XtAddCallback (messageBox, XmNokCallback,
		(XtCallbackProc) MYuditFindDialogFindCallback, this);
	XtAddCallback (messageBox, XmNcancelCallback,
		(XtCallbackProc) MYuditFindDialogReplaceCallback, this);
	XtAddCallback (messageBox, XmNhelpCallback,
		(XtCallbackProc) MYuditFindDialogDismissCallback, this);

	uWidgetParent = uwidgetIn;

	string = Translate ("Find String:");
	findLabel = XtVaCreateManagedWidget ("FindLabel",
		xmLabelWidgetClass, 	rowColumn,
		XtVaTypedArg,		XmNlabelString,		
		XmRString,		string, strlen (string) + 1,
		0);
	findForm = XtVaCreateManagedWidget ("FindForm",
		xmFormWidgetClass,	rowColumn,
		XmNresizable,		True,
		XmNwidth,		uWidgetParent->uMinimumWidth() * 8 ,
		XmNheight,		uWidgetParent->uMinimumHeight(),
		0);

	string = Translate ("Replace With:");
	replaceLabel = XtVaCreateManagedWidget ("ReplaceLabel",
		xmLabelWidgetClass, 	rowColumn,
		XtVaTypedArg,		XmNlabelString,		
		XmRString,		string, strlen (string) + 1,
		0);
	replaceForm = XtVaCreateManagedWidget ("ReplaceForm",
		xmFormWidgetClass,	rowColumn,
		XmNresizable,		True,
		XmNwidth,		uWidgetParent->uMinimumWidth() * 8 ,
		XmNheight,		uWidgetParent->uMinimumHeight(),
		0);

	replaceUWidget = new MUEntry ("Replace", replaceForm);
	findUWidget = new MUEntry ("Find", findForm);

} 

FindDialog::~FindDialog ()
{
	if (findLabel) XtDestroyWidget (findLabel);
	if (replaceLabel) XtDestroyWidget (replaceLabel);
	if (findUWidget) delete findUWidget;
	if (replaceUWidget) delete replaceUWidget;
	if (findForm) XtDestroyWidget (findForm);
	if (replaceForm) XtDestroyWidget (replaceForm);
	if (rowColumn) XtDestroyWidget (rowColumn);
	if (topForm) XtDestroyWidget (topForm);
	if (rowColumn) XtDestroyWidget (rowColumn);
	if (messageBox) XtDestroyWidget (messageBox);
	if (toplevel) XtDestroyWidget (toplevel);
}

void
FindDialog::findString ()
{
	const XInputMap*	xInputMap;
	const char*		xInputName;
	XtManageChild (messageBox);

	replaceUWidget->setupWidget (uWidgetParent);
	replaceUWidget->setMultiLine (UWidget::SINGLE);

	findUWidget->setupWidget (replaceUWidget);
	findUWidget->uSetEditable (UWidget::EDITABLE);

	// XInput can not be inherited
	xInputName = (const char*) topConfig->getChecked (Config::XINPUT);
	xInputMap = topConfig->getXInputMap (xInputName);
	if (xInputMap==0)
	{
		cerr << "error: can not get XInputMap '"
			<< xInputName << "'.\n";
	}
	else
	{
		replaceUWidget->uSetXInput (xInputName,
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
		findUWidget->uSetXInput (xInputName,
			(const char*) xInputMap->list[XInputMap::TYPE],
			(const char*) xInputMap->list[XInputMap::STYLE],
			(const char*) xInputMap->list[XInputMap::TEXT_TYPE],
			(const char*) xInputMap->list[XInputMap::ENCODING],
			(const char*) xInputMap->list[XInputMap::FONTMAP],
			atof ((const char*)xInputMap->list[XInputMap::VERSION]));
	}

	XtVaSetValues (messageBox, XmNinitialFocus, findForm, 0);
	XtPopup (toplevel, XtGrabNone);
	MWait4Confirm (messageBox);
}

void
MYuditFindDialogFindCallback (Widget w, FindDialog* dialog, 
	XmAnyCallbackStruct *cbs)
{
	const UCS2*  const* textBuffer;
	const int*	textSizes;

	textBuffer = dialog->findUWidget->uGetTextBuffer ();
	textSizes = dialog->findUWidget->uGetTextSizes ();
	if (textBuffer!=0)
	{
		dialog->findResult = dialog->uWidgetParent->uFindUCS2 (
			textBuffer[0], textSizes[0], (dialog->findResult==-1) ? 1 : 0);
	}
	else
	{
		dialog->findResult = -1;
	}
}

void
MYuditFindDialogReplaceCallback (Widget w, FindDialog* dialog,
				XmAnyCallbackStruct *cbs)
{
	const UCS2* const* textBuffer;
	const int*	textSizes;

	if (dialog->findResult<0) MYuditFindDialogFindCallback(w, dialog, cbs);
	if (dialog->findResult<0) return;

	textBuffer = dialog->replaceUWidget->uGetTextBuffer ();
	textSizes = dialog->replaceUWidget->uGetTextSizes ();
	if (textBuffer!=0)
	{
		dialog->uWidgetParent->uReplaceUCS2 (textBuffer[0], textSizes[0]);
	}
	else
	{
		dialog->uWidgetParent->uReplaceUCS2 (0, textSizes[0]);
	}
	dialog->findResult = -2;
}

void
MYuditFindDialogDismissCallback (Widget w, FindDialog* dialog,
				XmAnyCallbackStruct *cbs)
{
	XtPopdown (dialog->toplevel);
}
